From e127ca4d13c907e57d0b0a3638018021010ced0b Mon Sep 17 00:00:00 2001
From: Edward Wang <edward.c.wang@compdigitec.com>
Date: Sat, 6 Jul 2013 20:56:16 -0400
Subject: [PATCH] lib: allow disabling automatic advancement

---
 include/vlc/libvlc_media_list_player.h | 24 ++++++++++++++++++++++++
 lib/libvlc.sym                         |  2 ++
 lib/media_list_player.c                | 25 ++++++++++++++++++++++++-
 3 files changed, 50 insertions(+), 1 deletion(-)

diff --git a/include/vlc/libvlc_media_list_player.h b/include/vlc/libvlc_media_list_player.h
index c95ad97..4ab7cbd 100644
--- a/include/vlc/libvlc_media_list_player.h
+++ b/include/vlc/libvlc_media_list_player.h
@@ -89,6 +89,20 @@ LIBVLC_API void
     libvlc_media_list_player_retain( libvlc_media_list_player_t *p_mlp );
 
 /**
+ * Turns on or off automatic playback (on by default)
+ * If on, the media player will automatically advance to the next item.
+ * If off, the media player will not automatically continue to the item
+ * at the end of the current item, enabling manual control of the next
+ * item to play.
+ *
+ * \param p_mlp media player list object
+ * \param automatic true to turn on automatic playback, false otherwise
+ */
+LIBVLC_API void
+    libvlc_media_list_player_set_automatic_mode(
+                    libvlc_media_list_player_t* p_mlp, bool automatic );
+
+/**
  * Return the event manager of this media_list_player.
  *
  * \param p_mlp media list player instance
@@ -147,6 +161,16 @@ LIBVLC_API int
     libvlc_media_list_player_is_playing( libvlc_media_list_player_t * p_mlp );
 
 /**
+ * Retrives the current automatic playback status (on by default)
+ *
+ * \param p_mlp media player list object
+ * \return true if automatic playback is enabled, false otherwise
+ */
+LIBVLC_API bool
+    libvlc_media_list_player_get_automatic_mode(
+                    libvlc_media_list_player_t* p_mlp );
+
+/**
  * Get current libvlc_state of media list player
  *
  * \param p_mlp media list player instance
diff --git a/lib/libvlc.sym b/lib/libvlc.sym
index c19628a..942b8b7 100644
--- a/lib/libvlc.sym
+++ b/lib/libvlc.sym
@@ -93,6 +93,7 @@ libvlc_media_list_lock
 libvlc_media_list_media
 libvlc_media_list_new
 libvlc_media_list_player_event_manager
+libvlc_media_list_player_get_automatic_mode
 libvlc_media_list_player_get_state
 libvlc_media_list_player_is_playing
 libvlc_media_list_player_new
@@ -104,6 +105,7 @@ libvlc_media_list_player_play_item_at_index
 libvlc_media_list_player_previous
 libvlc_media_list_player_release
 libvlc_media_list_player_retain
+libvlc_media_list_player_set_automatic_mode
 libvlc_media_list_player_set_media_list
 libvlc_media_list_player_set_media_player
 libvlc_media_list_player_set_playback_mode
diff --git a/lib/media_list_player.c b/lib/media_list_player.c
index d0a0ce1..4df5499 100644
--- a/lib/media_list_player.c
+++ b/lib/media_list_player.c
@@ -68,6 +68,7 @@ struct libvlc_media_list_player_t
     libvlc_media_list_t *       p_mlist;
     libvlc_media_player_t *     p_mi;
     libvlc_playback_mode_t      e_playback_mode;
+    bool                        b_automatically_advance;
 };
 
 /* This is not yet exported by libvlccore */
@@ -329,7 +330,7 @@ media_player_reached_end(const libvlc_event_t * p_event, void * p_user_data)
     libvlc_media_list_player_t * p_mlp = p_user_data;
 
     vlc_mutex_lock(&p_mlp->mp_callback_lock);
-    if (!p_mlp->are_mp_callback_cancelled)
+    if(!p_mlp->are_mp_callback_cancelled && p_mlp->b_automatically_advance)
         set_relative_playlist_position_and_play(p_mlp, 1);
     vlc_mutex_unlock(&p_mlp->mp_callback_lock);
 }
@@ -475,6 +476,7 @@ libvlc_media_list_player_new(libvlc_instance_t * p_instance)
     libvlc_event_manager_register_event_type( p_mlp->p_event_manager,
             libvlc_MediaListPlayerStopped );
     p_mlp->e_playback_mode = libvlc_playback_mode_default;
+    p_mlp->b_automatically_advance = true;
 
     return p_mlp;
 }
@@ -545,6 +547,27 @@ libvlc_media_list_player_event_manager(libvlc_media_list_player_t * p_mlp)
 }
 
 /**************************************************************************
+ * get_automatic_mode (Public)
+ **************************************************************************/
+bool libvlc_media_list_player_get_automatic_mode(libvlc_media_list_player_t* p_mlp)
+{
+	lock(p_mlp);
+	bool b = p_mlp->b_automatically_advance;
+	unlock(p_mlp);
+	return b;
+}
+
+/**************************************************************************
+ * set_automatic_mode (Public)
+ **************************************************************************/
+void libvlc_media_list_player_set_automatic_mode(libvlc_media_list_player_t* p_mlp, bool automatic)
+{
+	lock(p_mlp);
+	p_mlp->b_automatically_advance = automatic;
+	unlock(p_mlp);
+}
+
+/**************************************************************************
  *        set_media_player (Public)
  **************************************************************************/
 void libvlc_media_list_player_set_media_player(libvlc_media_list_player_t * p_mlp, libvlc_media_player_t * p_mi)
-- 
1.8.3.2

